"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserId = exports.UserRoles = exports.UserScopes = exports.UserGroups = exports.User = void 0;
const common_1 = require("@nestjs/common");
exports.User = (0, common_1.createParamDecorator)((_data, ctx) => {
    return ctx.switchToHttp().getRequest().user;
});
/**
 * Parameter Decorator that is used to retrieve the ids of all
 * Keycloak groups with the given prefix the User is a member of.
 *
 * For example:
 * `function(@UserGroups('project') groups: number[])`
 *
 * @param groupName prefix for the Keycloak groups.
 *
 * @returns number[]: a list of the groups ids.
 *
 * @fritzforsit
 */
exports.UserGroups = (0, common_1.createParamDecorator)((groupName, ctx) => {
    try {
        const request = ctx.switchToHttp().getRequest();
        const groups = request.user.groups.filter((group) => {
            return group.startsWith(groupName);
        });
        const groupIds = groups.map((group) => {
            return parseInt(group.replace(groupName, ''));
        });
        return groupIds;
    }
    catch (error) {
        throw new common_1.InternalServerErrorException('Error fetching user groups');
    }
});
/**
 * Parameter Decorator that is used to retrieve all user resource scopes.
 * Must be run after a RessourceGuard is used without the @DefineScope decorator
 *
 * For example:
 * `function(@UserGScopes() scopes: string[])`
 *
 * @returns string[]: a list of all resource scopes the user has access to on the
 * current resource.
 *
 * @fritzforsit
 */
exports.UserScopes = (0, common_1.createParamDecorator)((data, ctx) => {
    try {
        const request = ctx.switchToHttp().getRequest();
        const scopes = request.scopes;
        return scopes;
    }
    catch (error) {
        throw new common_1.InternalServerErrorException('Error fetching user scopes');
    }
});
/**
 * Parameter Decorator that is used retrieve the current users Keycloak Roles.
 *
 * For example:
 * `function(@UserRoles() userRoles: string[])`
 *
 * @returns string[]: the users Keycloak roles
 *
 * @fritzforsit
 */
exports.UserRoles = (0, common_1.createParamDecorator)((data, ctx) => {
    try {
        const request = ctx.switchToHttp().getRequest();
        const roles = request.user.roles;
        return roles;
    }
    catch (error) {
        throw new common_1.InternalServerErrorException('Error fetching user scopes');
    }
});
/**
 * Parameter Decorator that is used retrieve the current users Keycloak id.
 *
 * For example:
 * `function(@UserId() userId: string)`
 *
 * @returns string: the Keycloak Id
 *
 * @fritzforsit
 */
exports.UserId = (0, common_1.createParamDecorator)((data, ctx) => {
    try {
        const request = ctx.switchToHttp().getRequest();
        const id = request.user.sub;
        return id;
    }
    catch (error) {
        throw new common_1.InternalServerErrorException('Error fetching users Keycloak id');
    }
});
//# sourceMappingURL=user.decorator.js.map